"""Contain all zeromq devices needed by OCT
"""
from __future__ import print_function, unicode_literals
import zmq


def forwarder(frontend, backend):
    """Simple pub/sub forwarder

    :param int frontend: fontend zeromq port
    :param int backend: backend zeromq port
    """
    try:
        context = zmq.Context()

        front_sub = context.socket(zmq.SUB)
        front_sub.bind("tcp://*:%d" % frontend)

        front_sub.setsockopt_string(zmq.SUBSCRIBE, "")

        back_pub = context.socket(zmq.PUB)
        back_pub.bind("tcp://*:%d" % backend)

        print("forwarder started, backend on port : %d\tfrontend on port: %d" % (backend, frontend))
        zmq.proxy(front_sub, back_pub)
    except Exception as e:
        print(e)
    finally:
        front_sub.close()
        back_pub.close()
        context.term()


def streamer(frontend, backend):
    """Simple push/pull streamer

    :param int frontend: fontend zeromq port
    :param int backend: backend zeromq port
    """
    try:
        context = zmq.Context()

        front_pull = context.socket(zmq.PULL)
        front_pull.set_hwm(0)
        front_pull.bind("tcp://*:%d" % frontend)

        back_push = context.socket(zmq.PUSH)
        back_push.bind("tcp://*:%d" % backend)

        print("streamer started, backend on port : %d\tfrontend on port: %d" % (backend, frontend))
        zmq.proxy(front_pull, back_push)
    except Exception as e:
        print(e)
    finally:
        front_pull.close()
        back_push.close()
        context.term()



from __future__ import print_function
import os
import shutil
import time
import uuid
from datetime import datetime
from oct.results.output import output as output_results

import oct.results.stats_handler as stats_handler
from oct.utilities.configuration import configure
from oct.core.hq import HightQuarter


def process_results(output_dir, config):
    """Process results and output them
    """
    print('\nanalyzing results...\n')
    res = output_results(output_dir, config)
    if res:
        print('created: %s/results.html\n' % output_dir)
    else:
        print('results cannot be processed')


def copy_config(project_path, output_dir):
    """Copy current config file to output directory
    """
    project_config = os.path.join(project_path, 'config.json')
    saved_config = os.path.join(output_dir, 'config.json')
    shutil.copy(project_config, saved_config)


def start_hq(output_dir, config, topic, is_master=True, **kwargs):
    """Start a HQ
    """
    hq = HightQuarter(output_dir, config, topic, **kwargs)
    if is_master:
        hq.wait_turrets(config.get("min_turrets", 1))
    hq.run()


def generate_output_path(args, project_path):
    """Generate default output directory
    """
    milisec = datetime.now().microsecond
    dirname = 'results_{}_{}'.format(time.strftime('%Y.%m.%d_%H.%M.%S', time.localtime()), str(milisec))
    return os.path.join(project_path, 'results', dirname)


def run(args):
    """Start an oct project

    :param Namespace args: the commande-line arguments
    """
    kwargs = vars(args)

    if 'func' in kwargs:
        del kwargs['func']

    project_path = kwargs.pop('project_path')
    config = configure(project_path)

    output_dir = kwargs.pop('output_dir', None) or generate_output_path(args, project_path)

    stats_handler.init_stats(output_dir, config)

    topic = args.publisher_channel or uuid.uuid4().hex
    print("External publishing topic is %s" % topic)

    start_hq(output_dir, config, topic, **kwargs)

    if not args.no_results:
        process_results(output_dir, config)

    copy_config(project_path, output_dir)
    print('done.\n')


def run_command(sp):
    """
    Main function to run oct tests.
    """
    parser = sp.add_parser('run', help="run an oct project")
    parser.add_argument('project_path', help="The project directory")
    parser.add_argument('-p', '--publisher-channel', dest='publisher_channel',
                        help='the channel for the external publisher',
                        default=None)
    parser.add_argument('--no-results', action='store_true',
                        help="if set, html report and graphs will not be generated")
    parser.add_argument('-o', '--output-dir', help="output directory for test results")
    parser.add_argument('--with-forwarder', action='store_true',
                        help="Set if HQ should connect to external forwarder")
    parser.add_argument('--forwarder-address',
                        help="with form ip:port. If not set and --with-forwarder flag present HQ will use default values")
    parser.set_defaults(func=run)



import os
import ujson

from oct.utilities.configuration import get_db_uri
from oct.results.models import Result, Turret, set_database, db


def init_stats(output_dir, config):
    """Init all required ressources for stats handling
    :param str output_dir: the output directory for the results
    :param dict config: the project configuration
    """

    try:
        os.makedirs(output_dir, 0o755)
    except OSError as e:
        print("ERROR: Can not create output directory: %s\n" % e)
        raise

    db_uri = get_db_uri(config, output_dir)

    set_database(db_uri, db, config)

    tables_to_create = [t for t in [Result, Turret] if not t.table_exists()]

    db.connect()
    db.create_tables(tables_to_create)


class StatsHandler(object):
    """This class will handle results and stats comming from the turrets"""
    def __init__(self, insert_limit=150):
        self.results = []
        self.insert_limit = insert_limit

    def write_result(self, data):
        """Write the results received to the database
        :param dict data: the data to save in database
        :return: None
        """
        data['custom_timers'] = ujson.dumps(data['custom_timers'])
        self.results.append(data)

        if len(self.results) >= 150:  # 150 rows for SQLite default limit
            with db.execution_context():
                with db.atomic():
                    Result.insert_many(self.results).execute()
                del self.results[:]

    def write_remaining(self):
        """Write the remaning stack content
        """
        if not self.results:
            return
        with db.execution_context():
            with db.atomic():
                Result.insert_many(self.results).execute()
        del self.results[:]



"""Start a streamer or a forwarder device for multi-hq tests
"""
from __future__ import print_function

from oct.core import devices


def start_device(name, frontend, backend):
    """Start specified device

    :param str name: name of the device, MUST match one of ['forwarder', 'streamer']
    :param int frontend: frontend bind port for device
    :param int backend: backend bind port for device
    """
    device = getattr(devices, name)
    device(frontend, backend)


def run_device(args):
    start_device(args.device, args.frontend, args.backend)


def run_device_command(sp):
    """
    Main function to run oct tests.
    """
    parser = sp.add_parser('run-device', help="run an oct device for multi-HQ tests")
    parser.add_argument('device', help="The project directory", choices=['forwarder', 'streamer'])
    parser.add_argument('-f', '--frontend', help="frontend port", type=int, required=True)
    parser.add_argument('-b', '--backend', help="backend port", type=int, required=True)

    parser.set_defaults(func=run_device)



import os
import six
import requests

from oct.utilities.newproject import from_template


def download_armory(args):
    plan_name = args.plan

    results = requests.get("http://armory.theghouls.io/get-plan/%s" % plan_name)
    plan_name = plan_name.split('/')[-1]
    if results.status_code != 200:
        print("No plan found with name %s" % plan_name)
        print("Status : %s" % results.status_code)
        return None
    results = results.json()
    if not results.get('plans'):
        print("No plan found with name %s" % plan_name)
        return
    download_url = results['plans'][0]['gh_tar_url']
    file_name = os.path.join("/", "tmp", plan_name + ".tar.gz")
    r = requests.get(download_url, allow_redirects=True)
    with open(file_name, 'wb') as f:
        for chunk in r.iter_content(chunk_size=1024):
            if chunk:
                f.write(chunk)
    args.template = file_name
    from_template(args)
    os.remove(file_name)


def from_armory(sp):
    if six.PY2:
        parser = sp.add_parser('from-armory', help="create project from armory template")
    else:
        parser = sp.add_parser('from-armory', help="create project from armory template", aliases=['armory'])
    parser.add_argument('name', type=str, help="new project name")
    parser.add_argument('plan')
    parser.set_defaults(func=download_armory)
